from infisical.models.api import CreateSecretDTO, SecretResponse
from requests import Session


def create_secret_req(api_request: Session, options: CreateSecretDTO) -> SecretResponse:
    response = api_request.post(
        url=f"/api/v3/secrets/{options.secret_name}",
        json={
            "workspaceId": options.workspace_id,
            "environment": options.environment,
            "type": options.type,
            "secretKeyCiphertext": options.secret_key_ciphertext,
            "secretKeyIV": options.secret_key_iv,
            "secretKeyTag": options.secret_key_tag,
            "secretValueCiphertext": options.secret_value_ciphertext,
            "secretValueIV": options.secret_value_iv,
            "secretValueTag": options.secret_value_tag,
            "secretPath": options.path,
        },
    )

    json_object = response.json()

    json_object["secret"]["workspace"] = options.workspace_id
    json_object["secret"]["environment"] = options.environment


    
    return SecretResponse.parse_obj(json_object)



from infisical.models.api import GetSecretsDTO, SecretsResponse
from requests import Session


def get_secrets_req(api_request: Session, options: GetSecretsDTO) -> SecretsResponse:
    """Send request again Infisical API to fetch secrets.
    See more information on https://infisical.com/docs/api-reference/endpoints/secrets/read

    :param api_request: The :class:`requests.Session` instance used to perform the request
    :param workspace_id: The ID of the workspace
    :param environment: The environment
    :return: Returns the API response as-is
    """

    response = api_request.get(
        "/api/v3/secrets",
        params={
            "environment": options.environment,
            "workspaceId": options.workspace_id,
            "secretPath": options.path,
            "include_imports": str(options.include_imports).lower()
        },
    )

    json_object = response.json()

    for obj in json_object['secrets']:
        obj["workspace"] = options.workspace_id
        obj["environment"] = options.environment

    for obj in json_object['imports']:
        for obj in obj['secrets']:
            obj["workspace"] = options.workspace_id
            obj["environment"] = options.environment

    data = SecretsResponse.parse_obj(json_object)

    return (data.secrets if data.secrets else [], data.imports if data.imports else [])
    



from infisical.api.models import GetServiceTokenKeyResponse
from requests import Session


def get_service_token_data_key_req(
    api_request: Session,
) -> GetServiceTokenKeyResponse:
    """Send request again Infisical API to fetch service token data v3 key.

    :param api_request: The :class:`requests.Session` instance used to perform the request
    :return: Returns the API response as-is
    """
    response = api_request.get("/api/v3/service-token/me/key")

    return GetServiceTokenKeyResponse.parse_obj(response.json())



from typing import List, Optional

from infisical.models.models import Secret
from pydantic import BaseModel
from typing_extensions import Literal


class GetSecretsDTO(BaseModel):
    workspace_id: str
    environment: str
    path: str
    include_imports: bool


class GetSecretDTO(BaseModel):
    secret_name: str
    workspace_id: str
    environment: str
    type: Literal["shared", "personal"]
    path: str


class CreateSecretDTO(BaseModel):
    secret_name: str
    workspace_id: str
    environment: str
    type: Literal["shared", "personal"]
    path: str
    secret_key_ciphertext: str
    secret_key_iv: str
    secret_key_tag: str
    secret_value_ciphertext: str
    secret_value_iv: str
    secret_value_tag: str


class UpdateSecretDTO(BaseModel):
    secret_name: str
    workspace_id: str
    environment: str
    type: Literal["shared", "personal"]
    path: str
    secret_value_ciphertext: str
    secret_value_iv: str
    secret_value_tag: str


class DeleteSecretDTO(BaseModel):
    secret_name: str
    workspace_id: str
    environment: str
    type: Literal["shared", "personal"]
    path: str


class SecretImport(BaseModel):
    secretPath: str
    folderId: str
    environment: str
    secrets: List[Secret]

class SecretsResponse(BaseModel):
    secrets: List[Secret]
    imports: Optional[List[SecretImport]]


class SecretResponse(BaseModel):
    secret: Secret



from typing import Dict, Optional, Union

from pydantic import BaseModel
from requests import Session
from typing_extensions import Literal

class WorkspaceConfig(BaseModel):
    workspace_id: str
    workspace_key: str

class ServiceTokenCredentials(BaseModel):
    service_token_key: str

class ServiceTokenV3Credentials(BaseModel):
    public_key: str
    private_key: str

class ClientConfig(BaseModel):
    auth_mode: Literal["service_token", "service_token_v3"]
    credentials: Union[ServiceTokenCredentials, ServiceTokenV3Credentials]
    workspace_config: Optional[WorkspaceConfig]
    cache_ttl: int



import os
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Union

from typing_extensions import Literal

if TYPE_CHECKING:
    from infisical import InfisicalClient

from infisical.logger import logger
from infisical.models.models import SecretBundle
from infisical.services.secret_service import SecretService


def get_all_secrets_helper(instance: "InfisicalClient", environment: str, path: str, include_imports: bool, attach_to_os_environ: bool):
    try:
        if not instance.client_config:
            raise Exception("Failed to find client config")

        if not instance.client_config.workspace_config:
            instance.client_config.workspace_config = (
                SecretService.populate_client_config(
                    api_request=instance.api_request,
                    client_config=instance.client_config,
                )
            )

        secret_bundles = SecretService.get_decrypted_secrets(
            api_request=instance.api_request,
            workspace_id=instance.client_config.workspace_config.workspace_id,
            environment=environment,
            path=path,
            workspace_key=instance.client_config.workspace_config.workspace_key,
            include_imports=include_imports
        )

        for secret_bundle in secret_bundles:
            cache_key = f"{secret_bundle.type}-{secret_bundle.secret_name}"
            instance.cache[cache_key] = secret_bundle
            if attach_to_os_environ:
                os.environ[secret_bundle.secret_name] = secret_bundle.secret_value

        return secret_bundles
    except Exception as exc:
        if instance.debug:
            logger.exception(exc)

    return [SecretService.get_fallback_secret(secret_name="")]


def get_secret_helper(
    instance: "InfisicalClient",
    secret_name: str,
    type: Literal["shared", "personal"],
    environment: str,
    path: str,
):
    cache_key = f"{type}-{secret_name}"
    cached_secret: Union[SecretBundle, None] = None
    try:
        if not instance.client_config:
            raise Exception("Failed to find client config")

        if not instance.client_config.workspace_config:
            instance.client_config.workspace_config = (
                SecretService.populate_client_config(
                    api_request=instance.api_request,
                    client_config=instance.client_config,
                )
            )

        cached_secret = instance.cache.get(cache_key)

        if cached_secret:
            current_time = datetime.now()
            cache_expiry_time = cached_secret.last_fetched_at + timedelta(
                seconds=instance.client_config.cache_ttl
            )

            if current_time < cache_expiry_time:
                if instance.debug:
                    print(f"Returning cached secret: {cached_secret.secret_name}")

                return cached_secret

        secret_bundle = SecretService.get_decrypted_secret(
            api_request=instance.api_request,
            secret_name=secret_name,
            workspace_id=instance.client_config.workspace_config.workspace_id,
            environment=environment,
            workspace_key=instance.client_config.workspace_config.workspace_key,
            type=type,
            path=path,
        )

        instance.cache[secret_name] = secret_bundle

        return secret_bundle

    except Exception as exc:
        if instance.debug:
            logger.exception(exc)

        if cached_secret:
            if instance.debug:
                print(f"Returning cached secret: {cached_secret}")

            return cached_secret

    return SecretService.get_fallback_secret(secret_name=secret_name)


def create_secret_helper(
    instance: "InfisicalClient",
    secret_name: str,
    secret_value: str,
    type: Literal["shared", "personal"],
    environment: str,
    path: str,
):
    try:
        if not instance.client_config:
            raise Exception("Failed to find client config")

        if not instance.client_config.workspace_config:
            instance.client_config.workspace_config = (
                SecretService.populate_client_config(
                    api_request=instance.api_request,
                    client_config=instance.client_config,
                )
            )

        secret_bundle = SecretService.create_secret(
            api_request=instance.api_request,
            secret_name=secret_name,
            secret_value=secret_value,
            workspace_id=instance.client_config.workspace_config.workspace_id,
            environment=environment,
            workspace_key=instance.client_config.workspace_config.workspace_key,
            type=type,
            path=path,
        )

        cache_key = f"{type}-{secret_name}"
        instance.cache[cache_key] = secret_bundle

        return secret_bundle
    except Exception as exc:
        if instance.debug:
            logger.exception(exc)

    return SecretService.get_fallback_secret(secret_name=secret_name)


def update_secret_helper(
    instance: "InfisicalClient",
    secret_name: str,
    secret_value: str,
    type: Literal["shared", "personal"],
    environment: str,
    path: str,
):
    try:
        if not instance.client_config:
            raise Exception("Failed to find client config")

        if not instance.client_config.workspace_config:
            instance.client_config.workspace_config = (
                SecretService.populate_client_config(
                    api_request=instance.api_request,
                    client_config=instance.client_config,
                )
            )

        secret_bundle = SecretService.update_secret(
            api_request=instance.api_request,
            secret_name=secret_name,
            secret_value=secret_value,
            workspace_id=instance.client_config.workspace_config.workspace_id,
            environment=environment,
            workspace_key=instance.client_config.workspace_config.workspace_key,
            type=type,
            path=path,
        )

        cache_key = f"{type}-{secret_name}"
        instance.cache[cache_key] = secret_bundle

        return secret_bundle
    except Exception as exc:
        if instance.debug:
            logger.exception(exc)

    return SecretService.get_fallback_secret(secret_name=secret_name)


def delete_secret_helper(
    instance: "InfisicalClient",
    secret_name: str,
    type: Literal["shared", "personal"],
    environment: str,
    path: str,
):
    try:
        if not instance.client_config:
            raise Exception("Failed to find client config")

        if not instance.client_config.workspace_config:
            instance.client_config.workspace_config = (
                SecretService.populate_client_config(
                    api_request=instance.api_request,
                    client_config=instance.client_config,
                )
            )

        secret_bundle = SecretService.delete_secret(
            api_request=instance.api_request,
            secret_name=secret_name,
            workspace_id=instance.client_config.workspace_config.workspace_id,
            environment=environment,
            workspace_key=instance.client_config.workspace_config.workspace_key,
            type=type,
            path=path,
        )

        cache_key = f"{type}-{secret_name}"
        instance.cache[cache_key] = secret_bundle

        return secret_bundle
    except Exception as exc:
        if instance.debug:
            logger.exception(exc)

    return SecretService.get_fallback_secret(secret_name=secret_name)



import os
from datetime import datetime
from typing import List

from infisical.api.create_secret import create_secret_req
from infisical.api.delete_secret import delete_secret_req
from infisical.api.get_secret import get_secret_req
from infisical.api.get_secrets import get_secrets_req
from infisical.api.get_service_token_data import get_service_token_data_req
from infisical.api.get_service_token_data_key import get_service_token_data_key_req
from infisical.api.update_secret import update_secret_req
from infisical.helpers.secrets import transform_secret_to_secret_bundle
from infisical.models.api import (
    CreateSecretDTO,
    DeleteSecretDTO,
    GetSecretDTO,
    GetSecretsDTO,
    UpdateSecretDTO,
)
from infisical.models.models import SecretBundle
from infisical.models.secret_service import ClientConfig, WorkspaceConfig

# from infisical.utils.crypto import decrypt_symmetric, encrypt_symmetric
from infisical.utils.crypto import (
    decrypt_symmetric_128_bit_hex_key_utf8,
    encrypt_symmetric_128_bit_hex_key_utf8,
    decrypt_asymmetric
)
from requests import Session
from typing_extensions import Literal


class SecretService:
    @staticmethod
    def populate_client_config(
        api_request: Session, client_config: ClientConfig
    ) -> WorkspaceConfig:
        if client_config.auth_mode == "service_token":
            service_token_details = get_service_token_data_req(api_request)
            workspace_key = decrypt_symmetric_128_bit_hex_key_utf8(
                ciphertext=service_token_details.encrypted_key,
                iv=service_token_details.iv,
                tag=service_token_details.tag,
                key=client_config.credentials.service_token_key,
            )

            return WorkspaceConfig(
                workspace_id=service_token_details.workspace,
                workspace_key=workspace_key,
            )
        
        if client_config.auth_mode == "service_token_v3":
            service_token_key_details = get_service_token_data_key_req(api_request)
            workspace_key = decrypt_asymmetric(
                ciphertext=service_token_key_details.key.encrypted_key,
                nonce=service_token_key_details.key.nonce,
                public_key=service_token_key_details.key.public_key,
                private_key=client_config.credentials.private_key
            )
            
            return WorkspaceConfig(
                workspace_id=service_token_key_details.key.workspace,
                workspace_key=workspace_key
            )

    @staticmethod
    def get_fallback_secret(secret_name: str) -> SecretBundle:
        return SecretBundle(
            secret_name=secret_name,
            secret_value=os.environ[secret_name],
            is_fallback=True,
            last_fetched_at=datetime.now(),
        )

    @staticmethod
    def get_decrypted_secrets(
        api_request: Session,
        workspace_key: str,
        workspace_id: str,
        environment: str,
        path: str,
        include_imports: bool
    ) -> List[SecretBundle]:
        options = GetSecretsDTO(
            workspace_id=workspace_id, environment=environment, path=path, include_imports=include_imports
        )

        encrypted_secrets, secret_imports = get_secrets_req(api_request, options)
        
        secret_bundles: List[SecretBundle] = []

        for encrypted_secret in encrypted_secrets:
            secret_name = decrypt_symmetric_128_bit_hex_key_utf8(
                ciphertext=encrypted_secret.secret_key_ciphertext,
                iv=encrypted_secret.secret_key_iv,
                tag=encrypted_secret.secret_key_tag,
                key=workspace_key,
            )
            
            secret_value = decrypt_symmetric_128_bit_hex_key_utf8(
                ciphertext=encrypted_secret.secret_value_ciphertext,
                iv=encrypted_secret.secret_value_iv,
                tag=encrypted_secret.secret_value_tag,
                key=workspace_key,
            )
            
            secret_bundles.append(
                transform_secret_to_secret_bundle(
                    secret=encrypted_secret,
                    secret_name=secret_name,
                    secret_value=secret_value,
                )
            )
        
        for secret_import in secret_imports:
            for encrypted_secret in secret_import.secrets:
                secret_name = decrypt_symmetric_128_bit_hex_key_utf8(
                    ciphertext=encrypted_secret.secret_key_ciphertext,
                    iv=encrypted_secret.secret_key_iv,
                    tag=encrypted_secret.secret_key_tag,
                    key=workspace_key,
                )
                
                secret_value = decrypt_symmetric_128_bit_hex_key_utf8(
                    ciphertext=encrypted_secret.secret_value_ciphertext,
                    iv=encrypted_secret.secret_value_iv,
                    tag=encrypted_secret.secret_value_tag,
                    key=workspace_key,
                )

                secret_bundles.append(
                    transform_secret_to_secret_bundle(
                        secret=encrypted_secret,
                        secret_name=secret_name,
                        secret_value=secret_value,
                    )
                )

        return secret_bundles

    @staticmethod
    def get_decrypted_secret(
        api_request: Session,
        secret_name: str,
        workspace_id: str,
        environment: str,
        workspace_key: str,
        type: Literal["shared", "personal"],
        path: str,
    ):
        options = GetSecretDTO(
            secret_name=secret_name,
            workspace_id=workspace_id,
            environment=environment,
            type=type,
            path=path,
        )

        encrypted_secret = get_secret_req(
            api_request,
            options,
        )

        secret_value = decrypt_symmetric_128_bit_hex_key_utf8(
            ciphertext=encrypted_secret.secret.secret_value_ciphertext,
            iv=encrypted_secret.secret.secret_value_iv,
            tag=encrypted_secret.secret.secret_value_tag,
            key=workspace_key,
        )

        return transform_secret_to_secret_bundle(
            secret=encrypted_secret.secret,
            secret_name=secret_name,
            secret_value=secret_value,
        )

    @staticmethod
    def create_secret(
        api_request: Session,
        workspace_key: str,
        workspace_id: str,
        environment: str,
        type: Literal["shared", "personal"],
        secret_name: str,
        secret_value: str,
        path: str,
    ):
        (
            secret_key_ciphertext,
            secret_key_iv,
            secret_key_tag,
        ) = encrypt_symmetric_128_bit_hex_key_utf8(
            plaintext=secret_name, key=workspace_key
        )

        (
            secret_value_ciphertext,
            secret_value_iv,
            secret_value_tag,
        ) = encrypt_symmetric_128_bit_hex_key_utf8(
            plaintext=secret_value, key=workspace_key
        )

        options = CreateSecretDTO(
            secret_name=secret_name,
            workspace_id=workspace_id,
            environment=environment,
            type=type,
            path=path,
            secret_key_ciphertext=secret_key_ciphertext,
            secret_key_iv=secret_key_iv,
            secret_key_tag=secret_key_tag,
            secret_value_ciphertext=secret_value_ciphertext,
            secret_value_iv=secret_value_iv,
            secret_value_tag=secret_value_tag,
        )

        encrypted_secret = create_secret_req(api_request, options)

        return transform_secret_to_secret_bundle(
            secret=encrypted_secret.secret,
            secret_name=secret_name,
            secret_value=secret_value,
        )

    @staticmethod
    def update_secret(
        api_request: Session,
        workspace_key: str,
        workspace_id: str,
        environment: str,
        type: Literal["shared", "personal"],
        secret_name: str,
        secret_value: str,
        path: str,
    ):
        (
            secret_value_ciphertext,
            secret_value_iv,
            secret_value_tag,
        ) = encrypt_symmetric_128_bit_hex_key_utf8(
            plaintext=secret_value, key=workspace_key
        )

        options = UpdateSecretDTO(
            secret_name=secret_name,
            workspace_id=workspace_id,
            environment=environment,
            type=type,
            secret_value_ciphertext=secret_value_ciphertext,
            secret_value_iv=secret_value_iv,
            secret_value_tag=secret_value_tag,
            path=path,
        )

        encrypted_secret = update_secret_req(api_request, options)
        return transform_secret_to_secret_bundle(
            secret=encrypted_secret.secret,
            secret_name=secret_name,
            secret_value=secret_value,
        )

    @staticmethod
    def delete_secret(
        api_request: Session,
        workspace_key: str,
        workspace_id: str,
        environment: str,
        type: Literal["shared", "personal"],
        path: str,
        secret_name: str,
    ):
        options = DeleteSecretDTO(
            secret_name=secret_name,
            workspace_id=workspace_id,
            environment=environment,
            type=type,
            path=path,
        )

        encrypted_secret = delete_secret_req(api_request, options)

        secret_value = decrypt_symmetric_128_bit_hex_key_utf8(
            ciphertext=encrypted_secret.secret.secret_value_ciphertext,
            iv=encrypted_secret.secret.secret_value_iv,
            tag=encrypted_secret.secret.secret_value_tag,
            key=workspace_key,
        )

        return transform_secret_to_secret_bundle(
            secret=encrypted_secret.secret,
            secret_name=secret_name,
            secret_value=secret_value,
        )



from typing import Any, Optional, Union
from urllib.parse import urljoin

import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry

DEFAULT_TIMEOUT = 40  # seconds


class BaseUrlSession(requests.Session):
    base_url = ""

    def __init__(self, base_url: Optional[str] = None) -> None:
        if base_url:
            self.base_url = base_url
        super().__init__()

    def request(
        self,
        method: Union[str, bytes],
        url: Any,
        *args: Any,
        **kwargs: Any,
    ) -> requests.Response:
        url = self.create_url(url)
        return super().request(method, url, *args, **kwargs)

    def prepare_request(self, request: requests.Request) -> requests.PreparedRequest:
        request.url = self.create_url(request.url)
        return super().prepare_request(request)

    def create_url(self, url: str) -> str:
        return urljoin(self.base_url, url)


class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(
        self, request: requests.PreparedRequest, *args: Any, **kwargs: Any
    ) -> requests.Response:
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, *args, **kwargs)


def get_http_client(
    base_url: Optional[str],
    retries: int = 3,
    backoff_factor: int = 1,
    timeout: int = DEFAULT_TIMEOUT,
) -> BaseUrlSession:
    """Returns a pre-configured :class:`requests.Session` with a optional ``base_url``
    and some sane options for timeout and retry handling.

    :param base_url: (optional) A base url used for each request made with this client
    :param retries: The number of retries to do if request fail, defaults to 3
    :param backoff_factor: A backoff factor to apply between attempts after the second try, defaults to 1
    :param timeout: The timeout in seconds, defaults to 40
    :return: A ready-to-use instance of :class:`requests.Session`
    """
    retry_strategy = Retry(
        total=retries,
        backoff_factor=backoff_factor,
        status_forcelist=[429, 500, 502, 503, 504],
    )
    adapter = TimeoutHTTPAdapter(max_retries=retry_strategy, timeout=timeout)

    http = BaseUrlSession(base_url=base_url)
    http.mount("https://", adapter)
    http.mount("http://", adapter)

    return http



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class GetServiceTokenDetailsResponse(BaseModel):
    id: str = Field(..., alias="id")
    name: str
    workspace: str = Field(..., alias="projectId")
    expires_at: Optional[datetime] = Field(None, alias="expiresAt")
    encrypted_key: str = Field(..., alias="encryptedKey")
    iv: str
    tag: str
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    #v: int = Field(..., alias="__v")

class KeyData(BaseModel):
    id: str = Field(..., alias="id")
    workspace: str
    encrypted_key: str = Field(..., alias="encryptedKey")
    public_key: str = Field(..., alias="publicKey")
    nonce: str

class GetServiceTokenKeyResponse(BaseModel):
    key: KeyData


import re

INFISICAL_URL = "https://app.infisical.com"
AUTH_MODE_SERVICE_TOKEN = "service_token"
AUTH_MODE_SERVICE_TOKEN_V3 = "service_token_v3"

SERVICE_TOKEN_REGEX = re.compile(r"(st\.[a-f0-9-]+\.[a-f0-9]+)\.([a-f0-9]+)")
